/*
Example 1

This example shows a very simple J1939 implementation.  It uses polling
to check for a message to light an LED and to send a message if a
button is pressed.

Both Node 0 and Node 1 should be programmed with the same code, except
that OTHER_NODE should be defined as the other nodes J1939 Address.

Application Maestro should be run with the following options changed
from their default values (in addition to NAME, Address, and bit rate
values):

Interrupts or Polling - Polling
*/
#include <p18cxxx.h>
#include "j1939.h"

// meus configs:
#if 1						// j declarado em main.c
#pragma config WDT = OFF
#pragma config PBADEN = OFF	//PORTB<4> and PORTB<1:0> Configured as Digital
//I/O Pins on Reset
#pragma config MCLRE = OFF
#pragma config LVP = OFF

#if 0
#pragma config OSC = IRCIO7	//Internal RC with OSC1 as RA7 and OSC2 as divide
//by 4 clock out
#else
#pragma config OSC = HS		//com o XTAL 20MHz
#endif
#endif
//
J1939_MESSAGE Msg;

// Define some arbitrary values.  They must agree with the other node's
// values.

#define OTHER_NODE		128
#define TURN_ON_LED		92	
#define TURN_OFF_LED	94	
void delay_ms(unsigned int ms);
void lcd_init();
void Data(char Value);
void Cmd(char Value);
void Send2Lcd( char Adr, rom const char *Lcd);
void main( void )
{
	ADCON1=0X07;
	TRISCbits.TRISC1 = 0;//en
	TRISCbits.TRISC0 = 0;//rs
	TRISD=0X00;       
	TRISEbits.TRISE0 = 0;//bzr
	PORTEbits.RE0=0;         /*  LATD (0 - 7)pins Config as Output                  */
	TRISBbits.TRISB1 = 0; //LED 1 
	TRISCbits.TRISC3 = 1;//SW 1	
	lcd_init() ;	delay_ms(5);
	Send2Lcd(0x80,"   rhydoLABZ    ");
	Send2Lcd(0xc0,"CAN Prot:Test129");
	delay_ms(750);
	Send2Lcd(0xC0,"                ");
	PORTBbits.RB1=0; 
    J1939_Initialization( TRUE );
	while (J1939_Flags.WaitingForAddressClaimContention)// Wait for address contention to time out
	J1939_Poll(5);
	// Now we know our address should be good, so start checking for
	// messages and switches.
	while (1)
	{
		Send2Lcd(0xC0,"Press SW1");	 		
		if(PORTCbits.RC3==0)
		{
			PORTEbits.RE0=1;delay_ms(50);PORTEbits.RE0=0;
			Send2Lcd(0xc0,"                ");
			while(PORTCbits.RC3==0);
			Msg.DataPage				= 0;
			Msg.Priority				= J1939_CONTROL_PRIORITY;
			Msg.DestinationAddress	= OTHER_NODE;
			Msg.DataLength				= 0;
			Msg.PDUFormat = TURN_ON_LED;
			while (J1939_EnqueueMessage( &Msg ) != RC_SUCCESS)
			J1939_Poll(5);
			Send2Lcd(0xC0,"Data Sent    ");	
			delay_ms(500);
		}
		while (RXQueueCount > 0)
		{
			PORTEbits.RE0=1;delay_ms(50);PORTEbits.RE0=0;delay_ms(50);
			PORTEbits.RE0=1;delay_ms(50);PORTEbits.RE0=0;delay_ms(50);
			Send2Lcd(0xc0,"Data Received");
			J1939_DequeueMessage( &Msg );
			if (Msg.PDUFormat == TURN_ON_LED)
			{
				PORTBbits.RB1=1;
			}
			else if (Msg.PDUFormat == TURN_OFF_LED)
			{
				PORTBbits.RB1=0;
			}
			delay_ms(500);delay_ms(500);
			PORTBbits.RB1=0; 
			Send2Lcd(0xc0,"               ");
		}
		// Since we dont accept the Commanded Address message,
		// the value passed here doesnt matter.
		J1939_Poll(20);
	}
}
void delay_ms(unsigned int ms)
{
	int i,count;
	for(i=1;i<=ms;i++)
	{
		count=249;
		while(count!=1)
		{
			count--;
		}
	}
}
void lcd_init() 
{	
	delay_ms(15);              /*  Minimum Delay To Power On LCD Module To Recieve Mode*/
	Cmd(0X30);	delay_ms(5);     /*  LCD Specification Commands                          */
	Cmd(0X30);	delay_ms(1);      /*  LCD Specification Commands                          */
	Cmd(0X30);	delay_ms(2);   /*  LCD Specification Commands                          */
	Cmd(0X38);                 /*  LCD Double Line Display Command                     */
	Cmd(0X06);                 /*  LCD Auto Increment Location Address Command         */
	Cmd(0X01);                 /*  LCD Display Clear Command                           */
	Cmd(0X0C);                  /*  LCD Display ON Command                              */
}
void Cmd(char Value)
{
	PORTD=Value;  
	PORTCbits.RC0=0;                     /*  RC0=0(R/W=0)	[Write Process])                */
	PORTCbits.RC1=1;                     /*  PORTCbits.RC1=1(Enable=1)	[Enable Line ON]                */
	delay_ms(4);                /*  Minimun Delay For Hold On Data                      */
	PORTCbits.RC1=0;                     /*  PORTCbits.RC1=0(Enable=0)	[Enable Line OFF]               */
}
void Data(char Value)
{
	PORTD=Value;
	PORTCbits.RC0=1;                      /*  RC5=1(RS=1)		[Data Registr Selection])       */
	PORTCbits.RC1=1;                     /*  PORTCbits.RC1=1(Enable=1)	[Enable Line ON]                */
	delay_ms(4);                /*  Minimun Delay For Hold On Data                      */
	PORTCbits.RC1=0;                     /*  PORTCbits.RC1=0(Enable=0)	[Enable Line OFF]               */
}
void Send2Lcd(char Adr, rom const char *Lcd)
{
	Cmd(Adr);
	while(*Lcd!='\0')	
	{	
		Data(*Lcd);		
		Lcd++;	
	}
}